----------------------------------------------------------------------------
----------------------------------------------------------------------------
--
-- Addon name: No Thank You!
-- Function: Autoreplies if someone asks for portal
-- Made by: Aradel - Cho'gall

----------------------------------------------------------------------------
----------------------------------------------------------------------------

----------------------------------------------------------------------------
-- Frames/Components
----------------------------------------------------------------------------
local NTY_FONT_STRING     =   nil;
local NTY_BTN_SAVE        =   nil;
local NTY_BTN_CANCEL      =   nil;
local NTY_EDIT_MSG_BOX    =   nil;

----------------------------------------------------------------------------
-- Constants / Statics
----------------------------------------------------------------------------
local NTY_vSlash = {};
local NTY_vDelegate = {};
local NTY_eString = {};

local NTY_INVALID_SLASH_CMD = 1;

local NTY_DEFAULT_MESSAGE = "--#No Thank You! Auto reply--  BUSY, Walk!";
local NTY_STR_MSG_PREFIX    =   "|cFFD86B93 > NTY: |r";
local NTY_STR_ERROR_PREFIX  =   "|cFFFF0000 > NTY: |r"

NTY_REPLY = "";
-----------------------------------a-----------------------------------------
-- Utility Functions
----------------------------------------------------------------------------
local function NTY_ErrorMessage(id)
	DEFAULT_CHAT_FRAME:AddMessage(NTY_STR_ERROR_PREFIX .. NTY_eString[id]);
end

local function NTY_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(NTY_STR_MSG_PREFIX .. msg);
end

----------------------------------------------------------------------------
-- Addon Functions
----------------------------------------------------------------------------
local function NTY_AddonEnable()
    HTT:RegisterEvent("CHAT_MSG_WHISPER");
    NTY_Print("Enabled");
end

local function NTY_AddonDisable()
    HTT:UnregisterEvent("CHAT_MSG_WHISPER");
    NTY_Print("Disabled");
end

local function NTY_OnAddonLoaded(addon)
    if(addon == "NoThankYou") then
        if (_G["NTY_REPLY"] == nil) then
            NTY_REPLY = NTY_DEFAULT_MESSAGE;
        else
            if((type(_G["NTY_REPLY"]) == "string")) then
                if(string.len(_G["NTY_REPLY"]) == 0) then
                    NTY_REPLY = NTY_DEFAULT_MESSAGE;
                else
                    NTY_REPLY = _G["NTY_REPLY"];
                end
            end
        end
    end
end

local function NTY_OnPlayerLogout()
    _G["NTY_REPLY"] = NTY_REPLY;
end

local function NTY_SetReply()
    NTY_REPLY = NTY_EDIT_MSG_BOX:GetText();
    NTY_EDIT_MSG_BOX:SetText("");

    HTT:Hide();
end

local function NTY_Show()
    HTT:Show();
	NTY_EDIT_MSG_BOX:SetText(NTY_REPLY);
end

local function NTY_Hide()
    HTT:Hide();
end

local function NTY_OnWhisper(msg, sender, ...)
    msg = string.lower(msg);
    if string.find(msg, "portal") or string.find(msg, "port") then
        SendChatMessage(NTY_REPLY, WHISPER, nil, sender);
    end
end

local function NTY_Help()
    NTY_Print("|cFF34CCFF- Available Commands |r");
    NTY_Print("show - Opens custom message dialog.");
    NTY_Print("hide - Hides custom message dialog.");
    NTY_Print("on - Enables the addon.");
    NTY_Print("off - Disables the addon.");
    NTY_Print("help - Displays a list of available commands.");
end

----------------------------------------------------------------------------
-- VTables/Delegates
----------------------------------------------------------------------------
NTY_vSlash["ON"]    =   NTY_AddonEnable;
NTY_vSlash["OFF"]   =   NTY_AddonDisable;
NTY_vSlash["SHOW"]  =   NTY_Show;
NTY_vSlash["HIDE"]  =   NTY_Hide;
NTY_vSlash["HELP"]  =   NTY_Help;

NTY_vDelegate["CHAT_MSG_WHISPER"]   =   NTY_OnWhisper;
NTY_vDelegate["ADDON_LOADED"]       =   NTY_OnAddonLoaded;
NTY_vDelegate["PLAYER_LOGOUT"]      =   NTY_OnPlayerLogout;

----------------------------------------------------------------------------
-- Initialization Functions
----------------------------------------------------------------------------

local function NTY_SetErrorStrings()
    NTY_eString = {}
    NTY_eString[NTY_INVALID_SLASH_CMD] = "Invalid Slash Command - '/nty help' for a list of available commands."
end

local function NTY_InitializeFrames()
	HTT:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	});
	
	HTT:SetBackdropColor(0.5, 0.5, 0.5, 0.5);
	
    NTY_FONT_STRING = HTT:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    NTY_FONT_STRING:SetPoint("TOP", HTT, 0, -20);
    NTY_FONT_STRING:SetText("No Thank You!");
    NTY_FONT_STRING:SetTextHeight(28);

    NTY_BTN_SAVE = CreateFrame("Button", "NTY_BTN_SAVE", HTT, "UIPanelButtonTemplate");
    NTY_BTN_SAVE:SetSize(80, 25);
    NTY_BTN_SAVE:SetText("Save");
    NTY_BTN_SAVE:SetPoint("BOTTOM", HTT, 40, 15);
    NTY_BTN_SAVE:SetScript("OnClick", NTY_SetReply);

    NTY_BTN_CANCEL = CreateFrame("Button", "NTY_BTN_CANCEL", HTT, "UIPanelButtonTemplate");
    NTY_BTN_CANCEL:SetSize(80, 25);
    NTY_BTN_CANCEL:SetText("Cancel");
    NTY_BTN_CANCEL:SetPoint("BOTTOM", HTT, -40, 15);
    NTY_BTN_CANCEL:SetScript("OnClick", NTY_Hide);

    NTY_EDIT_MSG_BOX = CreateFrame("EditBox", "NTY_EDIT_MSG_BOX", HTT, InputBoxTemplate);
    NTY_EDIT_MSG_BOX:SetFontObject("ChatFontNormal");
    NTY_EDIT_MSG_BOX:SetTextInsets(8, 8, 0, 0);
    NTY_EDIT_MSG_BOX:SetSize(260, 40);
    NTY_EDIT_MSG_BOX:SetPoint("CENTER", HTT, 0, 0);
    NTY_EDIT_MSG_BOX:SetAutoFocus(false);
    NTY_EDIT_MSG_BOX:SetCursorPosition(0);
	NTY_EDIT_MSG_BOX:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	});
    NTY_EDIT_MSG_BOX:SetBackdropColor(1,1,1,1);
end

function NTY_OnLoad()
    HTT:RegisterEvent("CHAT_MSG_WHISPER");
    HTT:RegisterEvent("ADDON_LOADED");
    HTT:RegisterEvent("PLAYER_LOGOUT");

    SlashCmdList["NOTHANKYOU"] = NTY_OnSlashCommand;
    SLASH_NOTHANKYOU1 = "/nty";

    NTY_SetErrorStrings();
    NTY_InitializeFrames();
end

----------------------------------------------------------------------------
-- Entry Points (Main)
----------------------------------------------------------------------------
function NTY_OnSlashCommand(msg)
    local key = string.upper(msg);

    if type(NTY_vSlash[key]) ~= "nil" then
        NTY_vSlash[key](msg);
    else
        NTY_ErrorMessage(NTY_INVALID_SLASH_CMD);
    end
end

function NTY_OnEvent(HTT, event, ...)
    NTY_vDelegate[event](...);
end